/**
 *
 * Carga del nuevo WebOffice estableciendo funciones callback de guardado en servidor
 * Las únicas funciones públicas son:
 *   init:         carga inicial de la configuración
 *   resize:       mantiene actualizado el tamaño del editor. Solo necesario si se muestra en diálogo JQuery
 *   openDocument: abre doumento en editor
 */

        

// Objeto global que da acceso a recursos del editor WebOffice
var weboffice = {

    editor:null,             	// Referencia al objeto webodf
    nombreFichero:null,			// Nombre del documento actualmente en edición
    idDocumentoEdicion:null, 	// ID del documento actualmente en edición
    saveOpenedFile:null,     	// Función que de ser invocada guarda en servidor el documento actualmente en edición
    saveError:null,          	// Variable que determina si hubo un error al guardar
    idCapa:null,             	// Capa HTML-DIV donde se muestra el editor
    idCapaSave:null,            // Capa HTML-DIV para el diálogo guardar/descartar/cancelar
    idCapaAlert: null,          // Dialogo JQuery para los mensajes de tipo "alert"
    canCloseEditor: false, 		// Variable que determina si se dan las condiciones para que el editor pueda cerrarse 
    urlBaseDownload: null,      // Variable con la URL base de descarga de documentos para ser editados
    urlBaseUpload: null,        // Variable con la URL base de subida de documentos una vez editados
    showEditorInDialog: true,   // Variable que determina si se muestra el editor en modo "dialog" o no


    /**
     * Carga el nuevo editor WebOffice 
     *   idDocumento => Identificador del documento a editar
     * Si el editor aún no existe, se crea uno nuevo.
     * Si el editor existe, se quita el documento previamente cargado y se carga el nuevo.
     */
    openDocument: function(idDocumento) {
   		urlDescarga = this.urlBaseDownload + idDocumento; // URL servlet descarga documentos
	   	if(this.editor) {
			this.editor.closeDocument(function(err) {if(err) this.showMessageDialog(err);})
      	    this.editor.openDocumentFromUrl(urlDescarga,function(err) {if(err) this.showMessageDialog(err);});
        }
        else {
            this.createEditor(urlDescarga);
        } 
        this.idDocumentoEdicion = idDocumento;
        this.canCloseEditor = false;
        if(this.showEditorInDialog) {
			$("#" + this.idCapa).dialog("open");
		}
    },


    /**
     * Función para controlar el evento de cierre de ventana del navegador cuando el editor no está en modo diálogo
     */  
	eventBeforeUnloadWindow: function (e) {
	    var confirmationMessage = "";
		(e || window.event).returnValue = confirmationMessage;     //Gecko + IE
		return confirmationMessage;                                //Webkit, Safari, Chrome etc.*/
	},


    /**
     * Función que inicia la carga del nuevo editor WebOffice 
     */   
    init: function(initConfig) {

        this.idCapa = initConfig.idCapa;
        this.idCapaSave = initConfig.idCapaSave;
        this.idCapaAlert = initConfig.idCapaAlert;
        this.urlBaseDownload = initConfig.urlBaseDownload;
        this.urlBaseUpload = initConfig.urlBaseUpload;
        this.showEditorInDialog = initConfig.showEditorInDialog;
        this.nombreFichero = initConfig.nombreFichero;

        if(this.showEditorInDialog) {
            $("#" + this.idCapa).dialog({
               resizable: false,
               autoOpen: false,
               height: 0.95*$(window).height(),
               width: 0.95*$(window).width(),
               modal: true,
               beforeClose: this.onCloseEditorDialog // Mirar siguiente función definida
            });
        }
		else {
			window.addEventListener("beforeunload", this.eventBeforeUnloadWindow);		
        }
    },

   
    /**
     * Función callback que se ejecuta cuando el usuario intenta cerrar el editor 
     * No llamar desde fuera
     */
    onCloseEditorDialog: function(event,ui) {
            sod = $("#" + weboffice.idCapaSave);
            wod = $("#" + weboffice.idCapa); 
            sod.html("<p align='center'>&iquest;Quiere guardar los cambios en el documento <i>" + weboffice.nombreFichero + "</i> antes de cerrar?</p>");
 			sod.dialog({
				resizable: true,
				width:500,
				modal: true,
				buttons: {
                    "Cancelar": function() {$(this).dialog("close");},
					"Cerrar sin guardar": function() {
						weboffice.canCloseEditor=true;
                        if(weboffice.showEditorInDialog) {
							wod.dialog("close");
                        }
                        else {
							window.removeEventListener("beforeunload", weboffice.eventBeforeUnloadWindow);
  							window.close();
                        }
                        $(this).dialog("close")
                    ;},
                    "Guardar": function() {
						weboffice.saveOpenedFile();
						weboffice.canCloseEditor=(weboffice.saveError==null);  // Se perite cerrar el editor si se guardan los cambios sin error
                        if(weboffice.showEditorInDialog) {
                            	wod.dialog("close");
                        }
                        else if(weboffice.canCloseEditor) {
							window.removeEventListener("beforeunload", weboffice.eventBeforeUnloadWindow);
							window.close();
						}
						$(this).dialog("close");
					}
				}
	    });;
	    return weboffice.canCloseEditor;
    },

    /**
     * Redimensiona adecuadamente el tamaño de la capa del editor conforme a la ventana del navegador 
     */   
    resize: function() {
        if(this.showEditorInDialog) {
	   $("#" + this.idCapa).dialog({height: 0.95*$(window).height(),width: 0.95*$(window).width()});
        }
    },


   /**
    * Función auxiliar para mostrar mensajes, solo con la finalidad de mejorar el aspecto de un simple alert.
    * Requiere de JQuery cargado para funcionar.
    */
   showMessageDialog: function(msg) {
       ad = $("#" + this.idCapaAlert); 
       ad.html("<p>" + msg + "</p>");
       ad.dialog({
          modal: true,
          buttons: {
	     Ok: function() {$(this).dialog("close");}
          }
       });
   },
   


   /**
    * Carga el nuevo editor WebOffice. Función auxiliar, no llamar desde fuera.
    *   urlDescarga => URL de descarga del documento a editar
    */

    createEditor: function(urlDescarga) {

     	var editorOptions = {
      	    allFeaturesEnabled: true,
      	    saveCallback: save
   	};	

        // Si no estamos en una capa, añadimos botón de cierre dentro del propio editor.
        if(!this.showEditorInDialog) {
	   editorOptions.closeCallback = this.onCloseEditorDialog;
	}

   
   	/**
    	* Función que se invocará al presionar el botón de guardar 
    	*/
   	function save() {
   	    function saveByteArrayLocally(err, data) {
                  weboffice.saveError = err;
                  if (err) {weboffice.showMessageDialog("Error al guardar documento: " + err);return;}
                  var mimetype = "application/vnd.oasis.opendocument.text";
                  blob = new Blob([data.buffer], {type: mimetype});
                
                  console.log(weboffice.urlBaseUpload + "<=" + blob);
                  
                  var formData = new FormData();
                  formData.append("doc", weboffice.idDocumentoEdicion);
                  formData.append("upload", blob, weboffice.nombreFichero);
                  $.ajax({
                     async: false,
		     url: weboffice.urlBaseUpload,  // URL servlet subida documentos
                     type: "POST",
                     data: formData,
                     processData: false,
                     contentType: false,
                     success: function(response) {console.log(response);weboffice.showMessageDialog("Se guardaron correctamente los cambios realizados");},
                     error: function(jqXHR, textStatus, errorMessage) {
			 weboffice.showMessageDialog("Se produjo un error al guardar los cambios: " + errorMessage);
			 weboffice.saveError=errorMessage;
		     }                                      
                  });

            }
            weboffice.editor.getDocumentAsByteArray(saveByteArrayLocally);
      	}
	
   	this.saveOpenedFile = save;

   	function onEditorCreated(err, e) {
            if (err) {weboffice.showMessageDialog("Error al cargar el editor: " + err);return;}
      	    weboffice.editor = e;
      	    weboffice.editor.openDocumentFromUrl(urlDescarga, function(err) {
            	if (err) {
        	   weboffice.showMessageDialog("Error al cargar la plantilla: " + err);
            	}
      	    });
   	}

   	Wodo.createTextEditor(this.idCapa, editorOptions, onEditorCreated);
    }

}; 
